@@ -4,21 +4,26 @@ the flask extension
 import datetime
 import itertools
 import logging
-import sys
 import time
 import warnings
+
 from functools import wraps
 
-import six
-from flask import request, current_app, g, Blueprint
+from typing import Callable
+from typing import Dict
+from typing import List
+from typing import Optional
+from typing import Set
+from typing import Union
+
+from flask import request, current_app, g, Blueprint, Flask, Response
 from limits.errors import ConfigurationError
-from limits.storage import storage_from_string, MemoryStorage
-from limits.strategies import STRATEGIES
+from limits.storage import storage_from_string, Storage, MemoryStorage
+from limits.strategies import STRATEGIES, RateLimiter
 from werkzeug.http import http_date, parse_date
 
 from flask_limiter.wrappers import Limit, LimitGroup
 from .errors import RateLimitExceeded
-from .util import get_ipaddr
 
 
 class C:
@@ -59,64 +64,65 @@ class Limiter(object):
     The :class:`Limiter` class initializes the Flask-Limiter extension.
 
     :param app: :class:`flask.Flask` instance to initialize the extension with.
-    :param list default_limits: a variable list of strings or callables
+    :param key_func: a callable that returns the domain to rate limit
+      by.
+    :param default_limits: a variable list of strings or callables
      returning strings denoting global limits to apply to all routes.
      :ref:`ratelimit-string` for  more details.
-    :param bool default_limits_per_method: whether default limits are applied
+    :param default_limits_per_method: whether default limits are applied
      per method, per route or as a combination of all method per route.
-    :param function default_limits_exempt_when: a function that should return
+    :param default_limits_exempt_when: a function that should return
      True/False to decide if the default limits should be skipped
-    :param function default_limits_deduct_when: a function that receives the
+    :param default_limits_deduct_when: a function that receives the
      current :class:`flask.Response` object and returns True/False to decide
      if a deduction should be made from the default rate limit(s)
-    :param list application_limits: a variable list of strings or callables
+    :param application_limits: a variable list of strings or callables
      returning strings for limits that are applied to the entire application
      (i.e a shared limit for all routes)
-    :param function key_func: a callable that returns the domain to rate limit
-      by.
-    :param bool headers_enabled: whether ``X-RateLimit`` response headers are
+    :param headers_enabled: whether ``X-RateLimit`` response headers are
      written.
-    :param str strategy: the strategy to use.
+    :param strategy: the strategy to use.
      Refer to :ref:`ratelimit-strategy`
-    :param str storage_uri: the storage location.
+    :param storage_uri: the storage location.
      Refer to :ref:`ratelimit-conf`
-    :param dict storage_options: kwargs to pass to the storage implementation
+    :param storage_options: kwargs to pass to the storage implementation
      upon instantiation.
-    :param bool auto_check: whether to automatically check the rate limit in
+    :param auto_check: whether to automatically check the rate limit in
      the before_request chain of the application. default ``True``
-    :param bool swallow_errors: whether to swallow errors when hitting a rate
+    :param swallow_errors: whether to swallow errors when hitting a rate
      limit. An exception will still be logged. default ``False``
-    :param list in_memory_fallback: a variable list of strings or callables
+    :param in_memory_fallback: a variable list of strings or callables
      returning strings denoting fallback limits to apply when the storage is
      down.
-    :param bool in_memory_fallback_enabled: simply falls back to in memory
+    :param in_memory_fallback_enabled: simply falls back to in memory
      storage when the main storage is down and inherits the original limits.
-    :param str retry_after: Allows configuration of how the value of the
+    :param retry_after: Allows configuration of how the value of the
      `Retry-After` header is rendered. One of `http-date` or `delta-seconds`.
-    :param str key_prefix: prefix prepended to rate limiter keys and app context global names.
+    :param key_prefix: prefix prepended to rate limiter keys and app context global names.
+    :param enabled: Whether the extension is enabled or not
     """
 
     def __init__(
         self,
-        app=None,
-        key_func=None,
-        global_limits=[],
-        default_limits=[],
+        app: Optional[Flask] = None,
+        key_func: Callable[[], str] = None,
+        global_limits: List[str] = [],
+        default_limits: List[str] = [],
         default_limits_per_method=False,
-        default_limits_exempt_when=None,
-        default_limits_deduct_when=None,
-        application_limits=[],
+        default_limits_exempt_when: Callable[[], bool] = None,
+        default_limits_deduct_when: Callable[[], bool] = None,
+        application_limits: List[str] = [],
         headers_enabled=False,
-        strategy=None,
-        storage_uri=None,
+        strategy: Optional[str] = None,
+        storage_uri: Optional[str] = None,
         storage_options={},
         auto_check=True,
         swallow_errors=False,
-        in_memory_fallback=[],
+        in_memory_fallback: List[str] = [],
         in_memory_fallback_enabled=False,
-        retry_after=None,
-        key_prefix="",
-        enabled=True,
+        retry_after: str = None,
+        key_prefix: str = "",
+        enabled: bool = True,
     ):
         self.app = app
         self.logger = logging.getLogger("flask-limiter")
@@ -132,28 +138,24 @@ class Limiter(object):
         self._in_memory_fallback_enabled = (
             in_memory_fallback_enabled or len(in_memory_fallback) > 0
         )
-        self._exempt_routes = set()
-        self._request_filters = []
+        self._exempt_routes: Set[str] = set()
+        self._request_filters: List[Callable[[], bool]] = []
         self._headers_enabled = headers_enabled
-        self._header_mapping = {}
+        self._header_mapping: Dict[int, str] = {}
         self._retry_after = retry_after
         self._strategy = strategy
         self._storage_uri = storage_uri
         self._storage_options = storage_options
         self._auto_check = auto_check
         self._swallow_errors = swallow_errors
-        if not key_func:
-            warnings.warn(
-                "Use of the default `get_ipaddr` function is discouraged."
-                " Please refer to"
-                " https://flask-limiter.readthedocs.org/#rate-limit-domain"
-                " for the recommended configuration",
-                UserWarning,
-            )
+
+        # No longer optional
+        assert key_func
+
         if global_limits:
             self.__raise_global_limits_warning()
 
-        self._key_func = key_func or get_ipaddr
+        self._key_func = key_func
         self._key_prefix = key_prefix
 
         for limit in set(global_limits + default_limits):
@@ -164,6 +166,7 @@ class Limiter(object):
                     )
                 ]
             )
+
         for limit in application_limits:
             self._application_limits.extend(
                 [
@@ -180,6 +183,7 @@ class Limiter(object):
                     )
                 ]
             )
+
         for limit in in_memory_fallback:
             self._in_memory_fallback.extend(
                 [
@@ -188,32 +192,35 @@ class Limiter(object):
                     )
                 ]
             )
-        self._route_limits = {}
-        self._dynamic_route_limits = {}
-        self._blueprint_limits = {}
-        self._blueprint_dynamic_limits = {}
-        self._blueprint_exempt = set()
-        self._storage = self._limiter = None
+        self._route_limits: Dict[str, List[Limit]] = {}
+        self._dynamic_route_limits: Dict[str, List[LimitGroup]] = {}
+        self._blueprint_limits: Dict[str, List[Limit]] = {}
+        self._blueprint_dynamic_limits: Dict[str, List[LimitGroup]] = {}
+        self._blueprint_exempt: Set[str] = set()
+        self._storage: Optional[Storage] = None
+        self._limiter: Optional[RateLimiter] = None
         self._storage_dead = False
-        self._fallback_limiter = None
+        self._fallback_limiter: Optional[RateLimiter] = None
         self.__check_backend_count = 0
         self.__last_check_backend = time.time()
-        self.__marked_for_limiting = {}
+        self.__marked_for_limiting: Dict[str, List[str]] = {}
 
         class BlackHoleHandler(logging.StreamHandler):
             def emit(*_):
                 return
 
         self.logger.addHandler(BlackHoleHandler())
+
         if app:
             self.init_app(app)
 
-    def init_app(self, app):
+    def init_app(self, app: Flask):
         """
         :param app: :class:`flask.Flask` instance to rate limit.
         """
         config = app.config
         self.enabled = config.setdefault(C.ENABLED, self.enabled)
+
         if not self.enabled:
             return
 
@@ -236,6 +243,7 @@ class Limiter(object):
             **self._storage_options
         )
         strategy = self._strategy or config.setdefault(C.STRATEGY, "fixed-window")
+
         if strategy not in STRATEGIES:
             raise ConfigurationError("Invalid rate limiting strategy %s" % strategy)
         self._limiter = STRATEGIES[strategy](self._storage)
@@ -262,6 +270,7 @@ class Limiter(object):
         self._key_prefix = self._key_prefix or config.get(C.KEY_PREFIX)
 
         app_limits = config.get(C.APPLICATION_LIMITS, None)
+
         if not self._application_limits and app_limits:
             self._application_limits = [
                 LimitGroup(
@@ -281,6 +290,7 @@ class Limiter(object):
             self.__raise_global_limits_warning()
 
         conf_limits = config.get(C.GLOBAL_LIMITS, config.get(C.DEFAULT_LIMITS, None))
+
         if not self._default_limits and conf_limits:
             self._default_limits = [
                 LimitGroup(
@@ -295,6 +305,7 @@ class Limiter(object):
                     None,
                 )
             ]
+
         for limit in self._default_limits:
             limit.per_method = self._default_limits_per_method
             limit.exempt_when = self._default_limits_exempt_when
@@ -303,6 +314,7 @@ class Limiter(object):
         self.__configure_fallbacks(app, strategy)
 
         # purely for backward compatibility as stated in flask documentation
+
         if not hasattr(app, "extensions"):
             app.extensions = {}  # pragma: no cover
 
@@ -318,6 +330,7 @@ class Limiter(object):
         config = app.config
         fallback_enabled = config.get(C.IN_MEMORY_FALLBACK_ENABLED, False)
         fallback_limits = config.get(C.IN_MEMORY_FALLBACK, None)
+
         if not self._in_memory_fallback and fallback_limits:
             self._in_memory_fallback = [
                 LimitGroup(
@@ -332,6 +345,7 @@ class Limiter(object):
                     None,
                 )
             ]
+
         if not self._in_memory_fallback_enabled:
             self._in_memory_fallback_enabled = (
                 fallback_enabled or len(self._in_memory_fallback) > 0
@@ -344,13 +358,16 @@ class Limiter(object):
     def __should_check_backend(self):
         if self.__check_backend_count > MAX_BACKEND_CHECKS:
             self.__check_backend_count = 0
+
         if time.time() - self.__last_check_backend > pow(2, self.__check_backend_count):
             self.__last_check_backend = time.time()
             self.__check_backend_count += 1
+
             return True
+
         return False
 
-    def check(self):
+    def check(self) -> None:
         """
         check the limits for the current request
 
@@ -358,7 +375,7 @@ class Limiter(object):
         """
         self.__check_request_limit(False)
 
-    def reset(self):
+    def reset(self) -> None:
         """
         resets the storage if it supports being reset
         """
@@ -369,7 +386,7 @@ class Limiter(object):
             self.logger.warning("This storage type does not support being reset")
 
     @property
-    def limiter(self):
+    def limiter(self) -> RateLimiter:
         if self._storage_dead and self._in_memory_fallback_enabled:
             return self._fallback_limiter
         else:
@@ -388,10 +405,11 @@ class Limiter(object):
     def __inject_headers(self, response):
         self.__check_conditional_deductions(response)
         current_limit = getattr(g, "%s_view_rate_limit" % self._key_prefix, None)
+
         if self.enabled and self._headers_enabled and current_limit:
             try:
                 window_stats = self.limiter.get_window_stats(*current_limit)
-                reset_in = 1 + window_stats[0]
+                reset_in: Union[int, float] = 1 + window_stats[0]
                 response.headers.add(
                     self._header_mapping[HEADERS.LIMIT], str(current_limit[0].amount)
                 )
@@ -405,9 +423,12 @@ class Limiter(object):
 
                 if existing_retry_after_header is not None:
                     # might be in http-date format
-                    retry_after = parse_date(existing_retry_after_header)
+                    retry_after: Union[float, datetime.datetime] = parse_date(
+                        existing_retry_after_header
+                    )
 
                     # parse_date failure returns None
+
                     if retry_after is None:
                         retry_after = time.time() + int(existing_retry_after_header)
 
@@ -423,7 +444,7 @@ class Limiter(object):
                     and http_date(reset_in)
                     or int(reset_in - time.time()),
                 )
-            except:  # noqa: E722
+            except Exception as e:  # noqa: E722
                 if self._in_memory_fallback_enabled and not self._storage_dead:
                     self.logger.warning(
                         "Rate limit storage unreachable - falling back to"
@@ -437,13 +458,15 @@ class Limiter(object):
                             "Failed to update rate limit headers. " "Swallowing error"
                         )
                     else:
-                        six.reraise(*sys.exc_info())
+                        raise e
+
         return response
 
     def __evaluate_limits(self, endpoint, limits):
         failed_limit = None
         limit_for_header = None
         limits_escape = []
+
         if not getattr(g, "%s_conditional_deductions" % self._key_prefix, None):
             setattr(g, "%s_conditional_deductions" % self._key_prefix, {})
 
@@ -457,10 +480,12 @@ class Limiter(object):
                 limit_scope += ":%s" % request.method
             limit_key = lim.key_func()
             args = [limit_key, limit_scope]
+
             if not all(args):
                 self.logger.error(
                     "Skipping limit: %s. Empty value found in parameters.", lim.limit
                 )
+
                 continue
 
             if self._key_prefix:
@@ -484,6 +509,7 @@ class Limiter(object):
                 )
                 failed_limit = lim
                 limit_for_header = [lim.limit] + args
+
                 break
 
             limits_escape.append([lim.limit] + args)
@@ -498,6 +524,7 @@ class Limiter(object):
         endpoint = request.endpoint or ""
         view_func = current_app.view_functions.get(endpoint, None)
         name = "%s.%s" % (view_func.__module__, view_func.__name__) if view_func else ""
+
         if (
             not request.endpoint
             or not (self.enabled and self.initialized)
@@ -508,7 +535,8 @@ class Limiter(object):
             or g.get("%s_rate_limiting_complete" % self._key_prefix)
         ):
             return
-        limits, dynamic_limits = [], []
+        limits: List[Limit] = []
+        dynamic_limits: List[Limit] = []
 
         # this is to ensure backward compatibility with behavior that
         # existed accidentally, i.e::
@@ -531,6 +559,7 @@ class Limiter(object):
         if not in_middleware or implicit_decorator:
             limits = name in self._route_limits and self._route_limits[name] or []
             dynamic_limits = []
+
             if name in self._dynamic_route_limits:
                 for lim in self._dynamic_route_limits[name]:
                     try:
@@ -541,6 +570,7 @@ class Limiter(object):
                             name,
                             e,
                         )
+
         if request.blueprint:
             if (
                 request.blueprint in self._blueprint_dynamic_limits
@@ -570,11 +600,13 @@ class Limiter(object):
                             request.blueprint,
                             e,
                         )
+
             if request.blueprint in self._blueprint_limits and not limits:
                 limits.extend(self._blueprint_limits[request.blueprint])
 
         try:
             all_limits = []
+
             if self._storage_dead and self._fallback_limiter:
                 if in_middleware and name in self.__marked_for_limiting:
                     pass
@@ -585,6 +617,7 @@ class Limiter(object):
                         self.__check_backend_count = 0
                     else:
                         all_limits = list(itertools.chain(*self._in_memory_fallback))
+
             if not all_limits:
                 route_limits = limits + dynamic_limits
                 all_limits = (
@@ -602,6 +635,7 @@ class Limiter(object):
                 before_request_context = (
                     in_middleware and name in self.__marked_for_limiting
                 )
+
                 if (
                     (explicit_limits_exempt or combined_defaults)
                     and not before_request_context
@@ -611,7 +645,8 @@ class Limiter(object):
             self.__evaluate_limits(endpoint, all_limits)
         except Exception as e:
             if isinstance(e, RateLimitExceeded):
-                six.reraise(*sys.exc_info())
+                raise e
+
             if self._in_memory_fallback_enabled and not self._storage_dead:
                 self.logger.warning(
                     "Rate limit storage unreachable - falling back to"
@@ -623,7 +658,7 @@ class Limiter(object):
                 if self._swallow_errors:
                     self.logger.exception("Failed to rate limit. Swallowing error")
                 else:
-                    six.reraise(*sys.exc_info())
+                    raise e
 
     def __limit_decorator(
         self,
@@ -645,6 +680,7 @@ class Limiter(object):
             is_route = not isinstance(obj, Blueprint)
             name = "%s.%s" % (obj.__module__, obj.__name__) if is_route else obj.name
             dynamic_limit, static_limits = None, []
+
             if callable(limit_value):
                 dynamic_limit = LimitGroup(
                     limit_value,
@@ -679,6 +715,7 @@ class Limiter(object):
                         name,
                         e,
                     )
+
             if isinstance(obj, Blueprint):
                 if dynamic_limit:
                     self._blueprint_dynamic_limits.setdefault(name, []).append(
@@ -688,6 +725,7 @@ class Limiter(object):
                     self._blueprint_limits.setdefault(name, []).extend(static_limits)
             else:
                 self.__marked_for_limiting.setdefault(name, []).append(obj)
+
                 if dynamic_limit:
                     self._dynamic_route_limits.setdefault(name, []).append(
                         dynamic_limit
@@ -702,6 +740,7 @@ class Limiter(object):
                     ):
                         self.__check_request_limit(False)
                         setattr(g, "%s_rate_limiting_complete" % self._key_prefix, True)
+
                     return current_app.ensure_sync(obj)(*a, **k)
 
                 return __inner
@@ -710,37 +749,38 @@ class Limiter(object):
 
     def limit(
         self,
-        limit_value,
-        key_func=None,
-        per_method=False,
-        methods=None,
-        error_message=None,
-        exempt_when=None,
-        override_defaults=True,
-        deduct_when=None,
-    ):
+        limit_value: Union[str, Callable[[], str]],
+        key_func: Callable[[], str] = None,
+        per_method: bool = False,
+        methods: List[str] = None,
+        error_message: str = None,
+        exempt_when: Callable[[], bool] = None,
+        override_defaults: bool = True,
+        deduct_when: Callable[[Response], bool] = None,
+    ) -> Callable:
         """
         decorator to be used for rate limiting individual routes or blueprints.
 
         :param limit_value: rate limit string or a callable that returns a
          string. :ref:`ratelimit-string` for more details.
-        :param function key_func: function/lambda to extract the unique
+        :param key_func: function/lambda to extract the unique
          identifier for the rate limit. defaults to remote address of the
          request.
-        :param bool per_method: whether the limit is sub categorized into the
+        :param per_method: whether the limit is sub categorized into the
          http method of the request.
-        :param list methods: if specified, only the methods in this list will
+        :param methods: if specified, only the methods in this list will
          be rate limited (default: None).
         :param error_message: string (or callable that returns one) to override
          the error message used in the response.
-        :param function exempt_when: function/lambda used to decide if the rate
+        :param exempt_when: function/lambda used to decide if the rate
          limit should skipped.
-        :param bool override_defaults:  whether the decorated limit overrides
+        :param override_defaults:  whether the decorated limit overrides
          the default limits. (default: True)
-        :param function deduct_when: a function that receives the current
+        :param deduct_when: a function that receives the current
          :class:`flask.Response` object and returns True/False to decide if a
          deduction should be done from the rate limit
         """
+
         return self.__limit_decorator(
             limit_value,
             key_func,
@@ -754,14 +794,14 @@ class Limiter(object):
 
     def shared_limit(
         self,
-        limit_value,
-        scope,
-        key_func=None,
-        error_message=None,
-        exempt_when=None,
+        limit_value: Union[str, Callable[[], str]],
+        scope: Union[str, Callable[[], str]],
+        key_func: Callable[[], str] = None,
+        error_message: str = None,
+        exempt_when: Callable[[], bool] = None,
         override_defaults=True,
-        deduct_when=None,
-    ):
+        deduct_when: Callable[[Response], bool] = None,
+    ) -> Callable:
         """
         decorator to be applied to multiple routes sharing the same rate limit.
 
@@ -769,19 +809,20 @@ class Limiter(object):
          string. :ref:`ratelimit-string` for more details.
         :param scope: a string or callable that returns a string
          for defining the rate limiting scope.
-        :param function key_func: function/lambda to extract the unique
+        :param key_func: function/lambda to extract the unique
          identifier for the rate limit. defaults to remote address of the
          request.
         :param error_message: string (or callable that returns one) to override
          the error message used in the response.
         :param function exempt_when: function/lambda used to decide if the rate
          limit should skipped.
-        :param bool override_defaults:  whether the decorated limit overrides
+        :param override_defaults:  whether the decorated limit overrides
          the default limits. (default: True)
-        :param function deduct_when: a function that receives the current
+        :param deduct_when: a function that receives the current
          :class:`flask.Response`  object and returns True/False to decide if a
          deduction should be done from the rate limit
         """
+
         return self.__limit_decorator(
             limit_value,
             key_func,
@@ -798,6 +839,7 @@ class Limiter(object):
         decorator to mark a view or all views in a blueprint as exempt from
         rate limits.
         """
+
         if not isinstance(obj, Blueprint):
             name = "%s.%s" % (obj.__module__, obj.__name__)
 
@@ -806,16 +848,18 @@ class Limiter(object):
                 return current_app.ensure_sync(obj)(*a, **k)
 
             self._exempt_routes.add(name)
+
             return __inner
         else:
             self._blueprint_exempt.add(obj.name)
 
-    def request_filter(self, fn):
+    def request_filter(self, fn: Callable) -> Callable:
         """
         decorator to mark a function as a filter to be executed
         to check if the request is exempt from rate limiting.
         """
         self._request_filters.append(fn)
+
         return fn
 
     def __raise_global_limits_warning(self):
